/* data with historic values of parameters 
   taken to equations */

DATA Work.TABLE_VALUES;
    INPUT i v1 v2 v3 v4 v5 v6 v7 v8 v9 v10;
    DATALINES;
    1    4.60    4.59    4.61    4.52    4.64    4.90    5.77    5.68    6.15    7.17
    2    4.58    4.59    4.61    4.52    4.64    4.90    5.77    5.68    6.16    7.17
    3    4.58    4.56    4.60    4.52    4.63    4.89    5.75    5.66    6.15    7.15
    ;        
RUN;
 
DATA Work.TABLE_MARGINS;
    INPUT i margin;
    DATALINES;
    1    0.59
    2    0.61
    3    0.54
    ;
RUN;

PROC PRINT data=Work.TABLE_VALUES;
    TITLE "Input data values";
RUN;
 
PROC PRINT data=Work.TABLE_MARGINS;
    TITLE "Margins to input values";
RUN;

%LET TABLE_NAME_VALUES =  work.Table_Values;
%LET TABLE_NAME_MARGINS = Work.Table_Margins;
%LET OBS_COUNT = 3;

proc optmodel;
    set WSET = 1..10;
    var weights {WSET} >= 0 <= 1.0;
    con sum_weights_100: sum {i in WSET} weights[i] = 1.0;
    *expand sum_wag_100;
 
/*** values to array ***/
 
    num NOBS = &OBS_COUNT;
    set VSET = 1..&OBS_COUNT; 
    num values { VSET, WSET}; 
 
*    READ DATA &TABLE_NAME_VALUES(OBS=&OBS_COUNT) into [i] values[i, 1]=v1 values[i, 2]=v2 
                                             values[i, 3]=v3 values[i, 4]=v4
                                             values[i, 5]=v5 values[i, 6]=v6
                                             values[i, 7]=v7 values[i, 8]=v8
                                             values[i, 9]=v9 values[i, 10]=v10;
    READ DATA &TABLE_NAME_VALUES(OBS=&OBS_COUNT) into [i] {j in WSET} <values[i,j]=col('v'||j)>;
    *print values;

/*** margins to array ***/
 
    set MSET = 1..NOBS;
    num margins{MSET};
    READ DATA &TABLE_NAME_MARGINS(OBS=&OBS_COUNT) into [i] margins=margin;
    *print margins;
 
/*** Minimize Standard Deviation of function 
        for each row
        r(i) = (sum{i in WGET} value[i] * weights[i]) - margin;
***/
 
*    min min_std = sqrt(
                      divide(
                          sum{i in VSET}(
                            /*function r(i) */
                            (sum{j in WSET} (values[i,j] * weights[j]) - margins[i])
                            -
                              /* avg of all r(i)*/
                            divide( 
                                sum{k in VSET}( 
                                    sum{j in WSET} (values[k,j] * weights[j]) - margins[k]
                                ), 
                                NOBS
                              )
                          )**2,
                          NOBS
                        )
                    );

	impvar r {i in VSET} = (sum {j in WSET} values[i,j] * weights[j]) - margins[i];
	impvar avg = (sum {k in VSET} r[k]) / NOBS;
	min min_std = sum {i in VSET} (r[i] - avg)**2;
	num std = sqrt(min_std.sol / NOBS);
 
/*** ***/ 
    solve;
	print min_std std;
    print weights;
/*** ***/

	num weights_sol {WSET};
	for {j in WSET} weights_sol[j] = weights[j].sol;

	/* snap to grid */
	set STEPS = 0..100 by 2;
	/* y[j,s] = 1 if weights[j] = s/100, y[j,s] = 0 otherwise */
	var y {WSET, STEPS} binary;
	con y_con {j in WSET}:
		sum {s in STEPS} y[j,s] = 1;
	con weights_y_link {j in WSET}:
		weights[j] = sum {s in STEPS} (s/100) * y[j,s];
	var AbsoluteDeviation {WSET} >= 0;
	con AbsoluteDeviation_con1 {j in WSET}:
		AbsoluteDeviation[j] >= weights_sol[j] - weights[j];
	con AbsoluteDeviation_con2 {j in WSET}:
		AbsoluteDeviation[j] >= -weights_sol[j] + weights[j];
	min TotalAbsoluteDeviation = sum {j in WSET} AbsoluteDeviation[j];

	solve;
    print weights_sol weights AbsoluteDeviation;
	print min_std std;
quit;
